@@ -163,8 +163,8 @@ class TestSchema(unittest.TestCase):
         self.assertEqual(schema.column_names("test"), ["x", "y"])
 
     def test_schema_get_column_type(self):
-        schema = MappingSchema({"a": {"b": "varchar"}})
-        self.assertEqual(schema.get_column_type("a", "b").this, exp.DataType.Type.VARCHAR)
+        schema = MappingSchema({"A": {"b": "varchar"}})
+        self.assertEqual(schema.get_column_type("a", "B").this, exp.DataType.Type.VARCHAR)
         self.assertEqual(
             schema.get_column_type(exp.Table(this="a"), exp.Column(this="b")).this,
             exp.DataType.Type.VARCHAR,
@@ -201,7 +201,7 @@ class TestSchema(unittest.TestCase):
     def test_schema_normalization(self):
         schema = MappingSchema(
             schema={"x": {"`y`": {"Z": {"a": "INT", "`B`": "VARCHAR"}, "w": {"C": "INT"}}}},
-            dialect="spark",
+            dialect="clickhouse",
         )
 
         table_z = exp.Table(this="z", db="y", catalog="x")
@@ -210,6 +210,29 @@ class TestSchema(unittest.TestCase):
         self.assertEqual(schema.column_names(table_z), ["a", "B"])
         self.assertEqual(schema.column_names(table_w), ["c"])
 
-        # Clickhouse supports both `` and "" for identifier quotes; sqlglot uses "" when generating sql
         schema = MappingSchema(schema={"x": {"`y`": "INT"}}, dialect="clickhouse")
         self.assertEqual(schema.column_names(exp.Table(this="x")), ["y"])
+
+        # Check that add_table normalizes both the table and the column names to be added / updated
+        schema = MappingSchema()
+        schema.add_table("Foo", {"SomeColumn": "INT", '"SomeColumn"': "DOUBLE"})
+        self.assertEqual(schema.column_names(exp.Table(this="fOO")), ["somecolumn", "SomeColumn"])
+
+        # Check that names are normalized to uppercase for Snowflake
+        schema = MappingSchema(schema={"x": {"foo": "int", '"bLa"': "int"}}, dialect="snowflake")
+        self.assertEqual(schema.column_names(exp.Table(this="x")), ["FOO", "bLa"])
+
+        # Check that switching off the normalization logic works as expected
+        schema = MappingSchema(schema={"x": {"foo": "int"}}, normalize=False, dialect="snowflake")
+        self.assertEqual(schema.column_names(exp.Table(this="x")), ["foo"])
+
+        # Check that the correct dialect is used when calling schema methods
+        schema = MappingSchema(schema={"[Fo]": {"x": "int"}}, dialect="tsql")
+        self.assertEqual(
+            schema.column_names("[Fo]"), schema.column_names("`Fo`", dialect="clickhouse")
+        )
+
+        # Check that all column identifiers are normalized to lowercase for BigQuery, even quoted
+        # ones. Also, ensure that tables aren't normalized, since they're case-sensitive by default.
+        schema = MappingSchema(schema={"Foo": {"`BaR`": "int"}}, dialect="bigquery")
+        self.assertEqual(schema.column_names("Foo"), ["bar"])
